#
# editArticleHtml.py ( Python 2.6 )
#
# This file is part of the AccuWebPublish project to publish journal articles on accu.org.
#
# Contributors: Martin Moene
#
# $Id: $
#

#
# EditCVuHtml is a partly-automated process to convert ACCU journal articles in
# PDF format to HTML for use on the ACCU website (http://accu.org/).
#
# This Python script converts HTML 3.2 generated from Adobe Acrobat 8 into
# clean HTML so that it displays as the middle column with an article on the
# ACCU Website.
#
# The body part of this HTML can then be used as input to the 'body' input field
# of the New/Edit Journal Articles form on the ACCU website.
#
# Usage:
# editArticleHtml --logging={loglevel} \
#             --adjust-headings={offset} \
#             --template={htmltemplatefile} \
#               [file...|dir...]
#
# Example:
# prompt>python editArticleHtml.py --template=article-template.html \
#                                  --adjust-headings=-2 \
#                                  --logging=debug \
#                                  CVu216-Wood-Fsharp.html
# [ToDo]
# - exception handling for different files (not only input)
# - exit value:
#   2: there where errors
#   1: there where warnings
#   0: the input file was processed witout warnings or errors.
#
# [Describe]
#

"""Edit ACCU journal html for publication on accu.org"""

__version_info__ = ( 0, 1, 0, 'alpha', 0 )
__version_date__ = '$Date: $'
__version__      = '$Revision: $'
__author__       = 'Martin Moene <m.j.moene@eld.physics.LeidenUniv.nl>'
__url__          = 'http://www.eld.physics.leidenuniv.nl/~moene'

import logging
import string
import glob
import sys
import os
import re

from optparse import OptionParser, OptionGroup


class Constants:

   """Module-level constants"""

   LOGLEVELS = {
       'debug': logging.DEBUG,
        'info': logging.INFO,
     'warning': logging.WARNING,
       'error': logging.ERROR,
    'critical': logging.CRITICAL }

   TAGS = [
      'anchor', 'a',
      'body'  , 'b',
      'div'   ,
      'font'  ,
      'h1'    , 'h2', 'h3', 'h4', 'h5', 'h6', 'html',
      'img'   , 'i',
      'pre'   , 'p',
      'table' , 'th', 'tr', 'td', 'tt' ]

   KEEPPROPTAGS = [ 'anchor', 'a', 'img' ]

   REMOVETAGS = [ 'div', 'font' ]

   # Note: in most cases <b> results from <tt>...</tt>, so we transform <b> to
   # indicate code; however check if there's any <b> that really is intended as such.
   DECORATETAGS = {
      'table':'table class="sidebartable"',
        'pre':'pre class="programlisting"',
         'tt': 'tt class="literal"',
          'b': 'tt class="code"' }


class Context:

   """Context for processing step."""

   def __init__( self ):
      self.hoffset  = 0
      self.template = None
      self.edit     = None


def versionString():
   """Version string such as '1.2.3'"""
   return '.'.join(map(str, __version_info__[0:3]))


def versionState():
   """Version state such as 'alpha', 'beta', 'final'"""
   return __version_info__[3]


def versionDate():
   """Version date such as 'Thu, 10 Jun 2010'"""
   return __version_date__[34:-3]


def main( ):

   """Parse commoandline and process given files."""

   progname = os.path.basename( sys.argv[0] )

   context = Context()

   parser = OptionParser(
      usage="Usage: %prog [options] file-or-directory...",
      version="%prog  {ver} {state}  ({date})".format( ver=versionString(), state=versionState(), date=versionDate() ),
      description=__doc__  )

   parser.add_option(   "-v", "--verbose",  action="store_true" , dest="verbose"  , help="show more messages" )
   parser.add_option(   "-q", "--quiet"  ,  action="store_true" , dest="quiet"    , help="show less messages" )
   parser.add_option(        "--logging" , metavar="LEVEL"      , dest="logging"  , default="warning"        , help="logging level: debug,info,error,critical [warning]" )
   parser.add_option( "--adjust-headings",  action="store"      , dest="hoffset"  , default=None, help="adjust heading level [+0]" )
   parser.add_option(       "--template" ,  action="store"      , dest="template" , default=None, help="filename for HTML template [none]" )
   parser.add_option(           "--edit" ,  action="store"      , dest="edit"     , default=None, help="filename for file with regex edit commands [none]" )

   (options, args) = parser.parse_args()

   context.hoffset  = options.hoffset
   context.template = options.template
   context.edit     = options.edit

   logger.setLevel( level=Constants.LOGLEVELS.get( options.logging, logging.NOTSET ) )
   logger.debug( "+++ Start of run" )
   logger.debug( "Options parsed" )

   logger.debug( "header-adjust: {level}".format( level=context.hoffset ) )
   logger.debug( "template: {template}".format( template=context.template ) )
   logger.debug( "args: {args}".format( args=args ) )

   #
   # issue helpful message, unless files are specified:
   #
   if len( args ) < 1:
      parser.error( "expecting one or more filenames; try option --help" )

   if options.verbose:
      pass

   if  not context.template:
      parser.error( "require html template file with @Article_Body@ to continue; try option --help" )

   try:
      logger.debug( "Start processing files" )
      count = 0;
      for arg in expand_if_dir( args ):
         for f in glob.glob( arg ):
            try:
               if not os.path.isfile( f ):
                  continue

               do_file( f, context )
               count += 1

            except IOError as (errno, strerror):
               print( "{progname}: cannot open file '{filename}', {strerror}".format( progname=progname, filename=f, strerror=strerror ) )

      if not options.quiet:
         if count <= 0:
            print( "%s: no files matched the given file specification(s)." % (progname) )
         else:
            print( "%s: %d file%s processed." % (progname, count, count != 1 and 's' or '') )

      logger.debug( "Finished processing files" )

   except Exception as detail:
      print( '{0}'.format( detail ) )
      logging.critical( '{0}'.format( detail ) )

   finally:
      logger.debug( "--- End of run" )


def expand_if_dir( arg ):
   """Expand directory into files, leave file as is."""
   new_arg = []
   for x in arg:
      if os.path.isdir( x ):
         new_arg.extend( glob.glob( os.path.join( x, '*.html' ) ) )
      else:
         new_arg.append( x )
   return new_arg


def do_file( filename, context ):
   """"Process given file."""
   try:
      root, ext = os.path.splitext( filename )
      outfilename = root + '-edited' + ext
      logger.info( "editing {input} to {output}".format( input=filename, output=outfilename ) )

      # read, edit input file
      with open( filename, 'r' ) as f:
         content = f.read()
         content = removeNewlines  ( content )
#         content = lowercaseHtml   ( content )
         content = cleanHtmlTags   ( content )
         content = removeHtmlTags  ( content )
         content = adjustHeadings  ( content, context.hoffset )
         content = decorateHtmlTags( content )
#         content = removeHtmlHead  ( content )
#         content = removeHtmlTail  ( content )
#         content = prettyFormat    ( content )
         content = editText        ( content, context.edit )

         writeOutput( content, context.template, outfilename )

   except CVuFileError as x:
      print( filename + ": skipped" )


def removeNewlines( content ):
   """remove newlines from input."""
   logger.debug( "stripping newlines" )
   return string.replace( content, "\n", " ", )


def lowercaseHtml( content ):
   """make HTML tags lowercase."""
   logger.debug( "html tags to lowercase" )
   for tag in Constants.TAGS:
      utag = tag.upper()
      content = string.replace( content, '<'  + utag, '<'  + tag )
      content = string.replace( content, '</' + utag, '</' + tag )
   return content


def cleanHtmlTags( content ):
   """for most tags, remove information following HTML tag."""
   logger.debug( "removing html tag extraneous information" )
   for tag in Constants.TAGS:
      # remove space before closing angle bracket:
      content = re.sub( r'\s+>', r'>', content )
      if  tag in Constants.KEEPPROPTAGS:
         continue
      # remove properties from tags (except <img...>):
      content = re.sub( r'<' + tag + ' [^>]+', r'<' + tag, content )
   return content


def removeHtmlTags( content ):
   """remove several tags completely."""
   logger.debug( "removing extraneous html tags" )
   for tag in Constants.REMOVETAGS:
      content = re.sub( r'<'  + tag + '>', r'', content )
      content = re.sub( r'</' + tag + '>', r'', content )
   return content


def adjustHeadings( content, hoffset ):
   """adjust level of headings."""
   if  None <> hoffset:
      adjust = int( hoffset )
      logger.debug( "adjusting level of headings" )
      if  adjust < 0:
         r = range( -adjust + 1, 6, +1 )
      else:
         r = range( 6 - adjust, 1, -1 )

      for i in r:
         hfrom = 'h' + str( i )
         hto   = 'h' + str( i + adjust )
         content = string.replace( content, '<'  + hfrom, '<'  + hto )
         content = string.replace( content, '</' + hfrom, '</' + hto )
   return content


def decorateHtmlTags( content ):
   """decorate several tags with style, e.g <table>, <pre>."""
   logger.debug( "decorating html tags" )
   for tag, newtag in Constants.DECORATETAGS.iteritems():
      content = re.sub( r'<' + tag + '>', r'<' + newtag + '>', content )
   return content


def removeHtmlHead( content ):
   """remove everything above and including <body>."""
   logger.debug( "removing html heading" )
   content = re.sub( r'^<.*<body>', r'', content )
   return content


def removeHtmlTail( content ):
   """remove everything below and including </body>."""
   logger.debug( "removing html trailer" )
   content = re.sub( r'</body></html>', r'', content )
   return content


def prettyFormat( content ):
   """Insert linebreaks at chapter titles, paragraphs etc.."""
   logger.debug( "pretty formatting file" )
   content =     re.sub( r'(<h[1-6]>[^>]+>)', r'\n\n\1\n\n', content )
   content =     re.sub( r'</p>[ ]*<p>', r'</p>\n\n<p>', content )
   content = string.replace( content, '<table', '\n\n<table' )
   content = string.replace( content, '</table>', '\n</table>' )
   content = string.replace( content, '<tr>', '\n<tr>' )
   return content


def editText( content, editPath ):
   """
   edit content based on file with reg.sub() fromexp, toexp.
   format: "abc" => "xyz"
   """
   logger.debug( "editing using re.sub() from file '{0}'".format(editPath) )
   if  None <> editPath:
#      print content
      # read file with edit ommands
      with open( editPath, 'r' ) as f:
         for line in f:
            line = line.strip()
            if  len( line ) > 1  and  -1 == line.find( '#', 0, 1 ):
               x = r'"(?P<from>.+)"\s*=>\s*"(?P<to>.*)"'
               m = re.search(x, line)
               fromexp = m.group('from')
               toexp = m.group('to')
#               print fromexp, '--->', toexp
               logger.debug( "edit: '{frm}' => '{to}'".format( frm=fromexp, to=toexp) )
               content = re.sub( fromexp, toexp, content )
   return content


def writeOutput( content, template, outfilename ):
   """create output file using template file."""
   logger.debug( "creating output" )
   with open( template, 'r' ) as t:
      newcontent = t.read()
      newcontent = string.replace( newcontent, '@Article_Body@', content )
      o = open( outfilename, 'w' );
      o.write( newcontent )


class CVuFileError( Exception ):
   def __init__( self, value ):
      self.value = value
   def __str__( self ):
      return repr( self.value )

# ---------------------------------------------------------------------------

if __name__ == "__main__":
   logging.basicConfig(
      level=logging.DEBUG,
      format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
      datefmt='%Y-%m-%d %H:%M:%S' )
   logger = logging.getLogger( 'CVu' )

   import sys
   main( )

#
# end of file
#
